@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -67,6 +67,17 @@ class TestNamedTraitObserverEqualHash(unittest.TestCase):
         imposter.optional = True
         self.assertNotEqual(observer, imposter)
 
+    def test_slots(self):
+        observer = NamedTraitObserver(name="foo", notify=True, optional=True)
+        with self.assertRaises(AttributeError):
+            observer.__dict__
+        with self.assertRaises(AttributeError):
+            observer.__weakref__
+
+    def test_eval_repr_roundtrip(self):
+        observer = NamedTraitObserver(name="foo", notify=True, optional=True)
+        self.assertEqual(eval(repr(observer)), observer)
+
 
 class TestObserverGraphIntegrateNamedTraitObserver(unittest.TestCase):
     """ Test integrating ObserverGraph with NamedTraitObserver as nodes.
@@ -434,6 +445,75 @@ class TestNamedTraitObserverTraitAdded(unittest.TestCase):
         foo.value += 1
         self.assertEqual(handler.call_count, 0)
 
+    def test_remove_trait_then_add_trait_again(self):
+        # Test a scenario where a trait exists when the observer is hooked,
+        # but then the trait is removed, and then added back again, the
+        # observer is gone, because the CTrait is gone and the trait_added
+        # event is not fired for something already defined on the class.
+
+        # given
+        # the trait exists, we can set optional to false.
+        graph = create_graph(
+            create_observer(name="value1", notify=True, optional=False),
+        )
+        handler = mock.Mock()
+        foo = ClassWithTwoValue()
+        call_add_or_remove_notifiers(
+            object=foo, graph=graph, handler=handler, remove=False)
+
+        # sanity check, the handler is called when the trait changes.
+        foo.value1 += 1
+        handler.assert_called_once()
+        handler.reset_mock()
+
+        # when
+        # remove the trait
+        foo.remove_trait("value1")
+
+        # then
+        # the handler is gone with the instance trait.
+        foo.value1 += 1
+        handler.assert_not_called()
+
+        # when
+        # Add the trait back...
+        foo.add_trait("value1", Int())
+
+        # then
+        # won't bring the handler back, because the 'value1' is defined as a
+        # class trait, trait_added is not fired when it is added.
+        foo.value1 += 1
+        handler.assert_not_called()
+
+    def test_add_trait_remove_trait_then_add_trait_again(self):
+        # Test a scenario when a trait is added, then removed, then added back.
+
+        # given
+        # trait is optional. It will be added later.
+        graph = create_graph(
+            create_observer(name="new_value", notify=True, optional=True),
+        )
+        handler = mock.Mock()
+        foo = ClassWithInstance()
+        call_add_or_remove_notifiers(
+            object=foo, graph=graph, handler=handler, remove=False)
+
+        foo.add_trait("new_value", Int())
+        foo.new_value += 1
+        handler.assert_called_once()
+        handler.reset_mock()
+
+        # when
+        # remove the trait and then add it back
+        foo.remove_trait("new_value")
+        foo.add_trait("new_value", Int())
+
+        # then
+        # the handler is now back! The trait was not defined on the class,
+        # so the last 'add_trait' fires a trait_added event.
+        foo.new_value += 1
+        handler.assert_called_once()
+
     def test_notifier_trait_added_distinguished(self):
         # Add two observers, both will have their own additional trait_added
         # observer. When one is removed, the other one is not affected.
